/**
 * Copyright Openmind http://www.openmindonline.it
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
package net.sourceforge.openutils.mgnlmedia.media.pages;

import info.magnolia.cms.core.Content;
import info.magnolia.cms.core.HierarchyManager;
import info.magnolia.cms.core.ItemType;
import info.magnolia.cms.core.MetaData;
import info.magnolia.cms.core.Path;
import info.magnolia.cms.core.SystemProperty;
import info.magnolia.cms.core.search.Query;
import info.magnolia.cms.core.search.QueryManager;
import info.magnolia.cms.core.search.QueryResult;
import info.magnolia.cms.exchange.ActivationManagerFactory;
import info.magnolia.cms.exchange.ExchangeException;
import info.magnolia.cms.security.Permission;
import info.magnolia.cms.util.AlertUtil;
import info.magnolia.cms.util.ExclusiveWrite;
import info.magnolia.commands.CommandsManager;
import info.magnolia.context.Context;
import info.magnolia.context.MgnlContext;
import info.magnolia.module.admininterface.commands.ActivationCommand;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.jcr.RepositoryException;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import net.sourceforge.openutils.mgnlmedia.media.configuration.MediaConfigurationManager;
import net.sourceforge.openutils.mgnlmedia.media.configuration.MediaTypeConfiguration;
import net.sourceforge.openutils.mgnlmedia.media.lifecycle.MediaModuleLifecycle;

import org.apache.commons.chain.Command;
import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;


/**
 * @author molaschi
 * @version $Id: MediaFolderViewPage.java 926 2008-11-10 15:40:59Z fgiust $
 */
public class MediaFolderViewPage extends MessagesTemplatedMVCHandler
{

    /**
     *
     */
    public static final String SORT_BY_FILENAME = "filename";

    /**
     *
     */
    public static final String SORT_BY_MODIFICATIONDATE = "modificationdate";

    /**
     *
     */
    public static final String SORT_BY_USAGE = "usage";

    /**
     * Logger.
     */
    private Logger log = LoggerFactory.getLogger(MediaFolderViewPage.class);

    private String path;

    private String node;

    private String dest;

    private String type;

    private String sorting;

    private boolean selectMedia;

    private String actMediaHandle;

    private String mediaType;

    private Collection<MediaBean> medias;

    private Collection<MediaTypeConfiguration> types;

    private Map<String, Integer> numberOfMedia;

    private boolean writable;

    private boolean canPublish;

    private boolean develop;

    /**
     * @param name
     * @param request
     * @param response
     */
    public MediaFolderViewPage(String name, HttpServletRequest request, HttpServletResponse response)
    {
        super(name, request, response);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void init()
    {
        super.init();

        if (mediaType != null && mediaType.length() > 0)
        {
            String[] mediaTypes = StringUtils.split(mediaType, ",");
            type = mediaTypes[0];
            types = new ArrayList<MediaTypeConfiguration>();
            for (String mt : mediaTypes)
            {
                types.add(MediaConfigurationManager.getInstance().getTypes().get(mt));
            }
        }
        else if (type == null)
        {
            type = MediaConfigurationManager.getInstance().getTypes().entrySet().iterator().next().getKey();
            types = MediaConfigurationManager.getInstance().getTypes().values();
        }
        else
        {
            types = MediaConfigurationManager.getInstance().getTypes().values();
        }

        fillNumberOfMediaPerType();

        if (sorting == null)
        {
            sorting = SORT_BY_MODIFICATIONDATE;
        }
    }

    private void fillNumberOfMediaPerType()
    {
        numberOfMedia = new HashMap<String, Integer>();
        for (MediaTypeConfiguration mtc : types)
        {
            QueryManager qm = MgnlContext.getQueryManager(MediaModuleLifecycle.REPO);
            try
            {
                Query q = qm.createQuery(path.substring(1) + "/*[type='" + mtc.getName() + "']", Query.XPATH);
                QueryResult qr = q.execute();

                numberOfMedia.put(mtc.getName(), qr.getContent(MediaConfigurationManager.MEDIA.getSystemName()).size());
            }
            catch (RepositoryException e)
            {

            }
        }
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public String show()
    {
        HierarchyManager hm = MgnlContext.getInstance().getHierarchyManager(MediaModuleLifecycle.REPO);

        try
        {
            Content folder = hm.getContent(this.getPath());

            writable = folder.isGranted(Permission.WRITE);

            canPublish = writable && ActivationManagerFactory.getActivationManager().hasAnyActiveSubscriber();

            develop = SystemProperty.getBooleanProperty("magnolia.develop");

            medias = new ArrayList<MediaBean>();

            MediaTypeConfiguration mtc = MediaConfigurationManager.getInstance().getTypes().get(type);
            Collection<Content> mediasOfType = MediaConfigurationManager.getInstance().getMediaNodes(
                folder,
                type,
                sorting);

            for (Content media : mediasOfType)
            {
                MediaBean mb = new MediaBean();
                mb.setContent(media);
                mb.setMetaData(media.getMetaData());
                mb.setHandle(media.getHandle());
                mb.setFilename(mtc.getHandler().getFilename(media));
                mb.setDescription(mtc.getHandler().getDescription(media));
                mb.setThumbnailUrl(mtc.getHandler().getThumbnailUrl(media));
                mb.setPreviewUrl(mtc.getHandler().getPreviewUrl(media));
                mb.setDescription(mtc.getHandler().getDescription(media));
                mb.setUuid(media.getUUID());

                mb.getUsedInWebPages().addAll(
                    MediaConfigurationManager.getInstance().getUsedInWebPages(media.getUUID()));

                medias.add(mb);
            }

        }
        catch (RepositoryException ex)
        {
            log.error("Exception caught", ex);
        }
        return super.show();
    }

    /**
     * delete a node
     * @return view
     */
    public String delete()
    {
        HierarchyManager hm = MgnlContext.getInstance().getHierarchyManager(MediaModuleLifecycle.REPO);

        try
        {
            synchronized (ExclusiveWrite.getInstance())
            {
                hm.delete(this.node);
                hm.save();
            }
        }
        catch (RepositoryException ex)
        {
            log.error("Exception deleting node {} from repository media", node, ex);
        }

        return show();
    }

    /**
     * moves a node
     * @return view
     */
    public String move()
    {
        String nodeName = StringUtils.substringAfterLast(node, "/");
        String destinationNode = dest + "/" + nodeName;
        try
        {
            copyMoveNode(node, destinationNode, true);
        }
        catch (ExchangeException e)
        {
            log.error("Exception deactivating node", e);
            AlertUtil.setMessage("Problem during deactivation");
        }
        catch (RepositoryException e)
        {
            log.error("Exception moving node", e);
            AlertUtil.setMessage("Problem during moving");
        }
        return show();
    }

    /**
     * copy a node
     * @return view
     */
    public String copy()
    {
        String nodeName = StringUtils.substringAfterLast(node, "/");
        String destinationNode = dest + "/" + nodeName;
        try
        {
            copyMoveNode(node, destinationNode, false);
        }
        catch (ExchangeException e)
        {
            log.error("Exception deactivating node", e);
            AlertUtil.setMessage("Problem during deactivation");
        }
        catch (RepositoryException e)
        {
            log.error("Exception copying node", e);
            AlertUtil.setMessage("Problem during copying");
        }
        return show();
    }

    /**
     * Copy or move a node (from AdminTreeMVCHandler
     * @param source source node
     * @param destination destination folder
     * @param move move or copy?
     * @return new content
     * @throws ExchangeException publication problem
     * @throws RepositoryException repository exception
     */
    public Content copyMoveNode(String source, String destination, boolean move) throws ExchangeException,
        RepositoryException
    {
        HierarchyManager hm = MgnlContext.getHierarchyManager(MediaModuleLifecycle.REPO);

        String goTo = destination;

        if (hm.isExist(destination))
        {
            String parentPath = StringUtils.substringBeforeLast(destination, "/"); //$NON-NLS-1$
            String label = StringUtils.substringAfterLast(destination, "/"); //$NON-NLS-1$
            label = Path.getUniqueLabel(hm, parentPath, label);
            goTo = parentPath + "/" + label; //$NON-NLS-1$
        }
        if (move)
        {
            if (destination.indexOf(source + "/") == 0)
            {
                // todo: disable this possibility in javascript
                // move source into destinatin not possible
                return null;
            }
            // TODO verificare se funziona lo stesso
            // this.deactivateNode(source);
            try
            {
                hm.moveTo(source, goTo);
            }
            catch (Exception e)
            {
                // try to move below node data
                return null;
            }
        }
        else
        {
            // copy
            hm.copyTo(source, goTo);
        }
        Content newContent = hm.getContent(destination);
        try
        {
            newContent.updateMetaData();
            newContent.getMetaData().setUnActivated();
        }
        catch (Exception e)
        {
            if (log.isDebugEnabled())
            {
                log.debug("Exception caught: " + e.getMessage(), e); //$NON-NLS-1$
            }
        }
        newContent.save();
        return newContent;
    }

    @SuppressWarnings("unchecked")
    public String activate()
    {
        Command cmd = CommandsManager.getInstance().getCommand(CommandsManager.DEFAULT_CATALOG, "activateMedia");

        ActivationCommand actCmd = (ActivationCommand) cmd;

        StringBuffer sb = new StringBuffer();
        sb.append(MediaConfigurationManager.MEDIA.getSystemName());
        sb.append(",");
        sb.append(ItemType.CONTENTNODE.getSystemName());
        actCmd.setItemTypes(sb.toString());

        Context context = MgnlContext.getInstance();
        context.put(Context.ATTRIBUTE_REPOSITORY, MediaModuleLifecycle.REPO);
        context.put(Context.ATTRIBUTE_PATH, this.node);
        context.put(Context.ATTRIBUTE_RECURSIVE, false);

        try
        {
            cmd.execute(context);
        }
        catch (Exception e)
        {
            log.error("Exception activating media", e);
        }

        return this.show();
    }

    @SuppressWarnings("unchecked")
    public String deactivate()
    {
        Command cmd = CommandsManager.getInstance().getCommand(CommandsManager.DEFAULT_CATALOG, "deactivate");

        Context context = MgnlContext.getInstance();
        context.put(Context.ATTRIBUTE_REPOSITORY, MediaModuleLifecycle.REPO);
        context.put(Context.ATTRIBUTE_PATH, this.node);

        try
        {
            cmd.execute(context);
        }
        catch (Exception e)
        {
            log.error("Exception activating media", e);
        }

        return this.show();
    }

    public String download()
    {

        HierarchyManager hm = MgnlContext.getHierarchyManager(MediaModuleLifecycle.REPO);

        Content media;
        try
        {
            media = hm.getContent(path);
        }
        catch (RepositoryException e)
        {
            log.error("Error downloading media " + path, e);
            return null;
        }

        MediaTypeConfiguration mtc = MediaConfigurationManager.getInstance().getMediaTypeConfigurationFromMedia(media);
        String url = mtc.getHandler().getUrl(media);

        String filename = mtc.getHandler().getFullFilename(media);

        this.response.addHeader("Content-Disposition", "attachment; filename=" + filename);
        try
        {
            this.request.getRequestDispatcher(url).forward(request, response);
        }
        catch (ServletException e)
        {
            log.error("Error downloading media " + path, e);
        }
        catch (IOException e)
        {
            log.error("Error downloading media " + path, e);
        }

        return null;
    }

    /**
     * Returns the path.
     * @return the path
     */
    public String getPath()
    {
        return path;
    }

    /**
     * Returns the medias.
     * @return the medias
     */
    public Collection<MediaBean> getMedias()
    {
        return medias;
    }

    /**
     * Sets the medias.
     * @param medias the medias to set
     */
    public void setMedias(Collection<MediaBean> medias)
    {
        this.medias = medias;
    }

    /**
     * Sets the path.
     * @param path the path to set
     */
    public void setPath(String path)
    {
        this.path = path;
    }

    /**
     * Returns the types.
     * @return the types
     */
    public Collection<MediaTypeConfiguration> getTypes()
    {
        return types;
    }

    /**
     * Sets the types.
     * @param types the types to set
     */
    public void setTypes(Collection<MediaTypeConfiguration> types)
    {
        this.types = types;
    }

    /**
     * Returns the type.
     * @return the type
     */
    public String getType()
    {
        return type;
    }

    /**
     * Sets the type.
     * @param type the type to set
     */
    public void setType(String type)
    {
        this.type = type;
    }

    /**
     * @author molaschi
     * @version $Id: MediaFolderViewPage.java 926 2008-11-10 15:40:59Z fgiust $
     */
    public class MediaBean
    {

        private String handle;

        private String filename;

        private String description;

        private String thumbnailUrl;

        private String previewUrl;

        private Content content;

        private MetaData metaData;

        private String uuid;

        private List<String> usedInWebPages = new ArrayList<String>();

        /**
         * Returns the metaData.
         * @return the metaData
         */
        public MetaData getMetaData()
        {
            return metaData;
        }

        /**
         * Sets the metaData.
         * @param metaData the metaData to set
         */
        public void setMetaData(MetaData metaData)
        {
            this.metaData = metaData;
        }

        /**
         * Returns the handle.
         * @return the handle
         */
        public String getHandle()
        {
            return handle;
        }

        /**
         * Sets the handle.
         * @param handle the handle to set
         */
        public void setHandle(String handle)
        {
            this.handle = handle;
        }

        /**
         * Returns the filename.
         * @return the filename
         */
        public String getFilename()
        {
            return filename;
        }

        /**
         * Sets the filename.
         * @param filename the filename to set
         */
        public void setFilename(String filename)
        {
            this.filename = filename;
        }

        /**
         * Returns the description.
         * @return the description
         */
        public String getDescription()
        {
            return description;
        }

        /**
         * Sets the description.
         * @param description the description to set
         */
        public void setDescription(String description)
        {
            this.description = description;
        }

        /**
         * Returns the thumbnailUrl.
         * @return the thumbnailUrl
         */
        public String getThumbnailUrl()
        {
            return thumbnailUrl;
        }

        /**
         * Sets the thumbnailUrl.
         * @param thumbnailUrl the thumbnailUrl to set
         */
        public void setThumbnailUrl(String thumbnailUrl)
        {
            this.thumbnailUrl = thumbnailUrl;
        }

        /**
         * Returns the previewUrl.
         * @return the previewUrl
         */
        public String getPreviewUrl()
        {
            return previewUrl;
        }

        /**
         * Sets the previewUrl.
         * @param previewUrl the previewUrl to set
         */
        public void setPreviewUrl(String previewUrl)
        {
            this.previewUrl = previewUrl;
        }

        /**
         * Returns the content.
         * @return the content
         */
        public Content getContent()
        {
            return content;
        }

        /**
         * Sets the content.
         * @param content the content to set
         */
        public void setContent(Content content)
        {
            this.content = content;
        }

        /**
         * Returns the uuid.
         * @return the uuid
         */
        public String getUuid()
        {
            return uuid;
        }

        /**
         * Sets the uuid.
         * @param uuid the uuid to set
         */
        public void setUuid(String uuid)
        {
            this.uuid = uuid;
        }

        /**
         * Returns the usedInWebPages.
         * @return the usedInWebPages
         */
        public List<String> getUsedInWebPages()
        {
            return usedInWebPages;
        }

        /**
         * Sets the usedInWebPages.
         * @param usedInWebPages the usedInWebPages to set
         */
        public void setUsedInWebPages(List<String> usedInWebPages)
        {
            this.usedInWebPages = usedInWebPages;
        }

    }

    /**
     * Returns the writable.
     * @return the writable
     */
    public boolean isWritable()
    {
        return writable;
    }

    /**
     * Sets the writable.
     * @param writable the writable to set
     */
    public void setWritable(boolean writable)
    {
        this.writable = writable;
    }

    /**
     * Returns the canPublish.
     * @return the canPublish
     */
    public boolean isCanPublish()
    {
        return canPublish;
    }

    /**
     * Sets the canPublish.
     * @param canPublish the canPublish to set
     */
    public void setCanPublish(boolean canPublish)
    {
        this.canPublish = canPublish;
    }

    /**
     * Returns the node.
     * @return the node
     */
    public String getNode()
    {
        return node;
    }

    /**
     * Sets the node.
     * @param node the node to set
     */
    public void setNode(String node)
    {
        this.node = node;
    }

    /**
     * Returns the dest.
     * @return the dest
     */
    public String getDest()
    {
        return dest;
    }

    /**
     * Sets the dest.
     * @param dest the dest to set
     */
    public void setDest(String dest)
    {
        this.dest = dest;
    }

    /**
     * Returns the selectMedia.
     * @return the selectMedia
     */
    public boolean isSelectMedia()
    {
        return selectMedia;
    }

    /**
     * Sets the selectMedia.
     * @param selectMedia the selectMedia to set
     */
    public void setSelectMedia(boolean selectMedia)
    {
        this.selectMedia = selectMedia;
    }

    /**
     * Returns the actMediaHandle.
     * @return the actMediaHandle
     */
    public String getActMediaHandle()
    {
        return actMediaHandle;
    }

    /**
     * Sets the actMediaHandle.
     * @param actMediaHandle the actMediaHandle to set
     */
    public void setActMediaHandle(String actMediaHandle)
    {
        this.actMediaHandle = actMediaHandle;
    }

    /**
     * Returns the mediaType.
     * @return the mediaType
     */
    public String getMediaType()
    {
        return mediaType;
    }

    /**
     * Sets the mediaType.
     * @param mediaType the mediaType to set
     */
    public void setMediaType(String mediaType)
    {
        this.mediaType = mediaType;
    }

    /**
     * Returns the numberOfMedia.
     * @return the numberOfMedia
     */
    public Map<String, Integer> getNumberOfMedia()
    {
        return numberOfMedia;
    }

    /**
     * Sets the numberOfMedia.
     * @param numberOfMedia the numberOfMedia to set
     */
    public void setNumberOfMedia(Map<String, Integer> numberOfMedia)
    {
        this.numberOfMedia = numberOfMedia;
    }

    /**
     * Returns the develop.
     * @return the develop
     */
    public boolean isDevelop()
    {
        return develop;
    }

    /**
     * Sets the develop.
     * @param develop the develop to set
     */
    public void setDevelop(boolean develop)
    {
        this.develop = develop;
    }

}
