/**
 * Copyright Openmind http://www.openmindonline.it
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
package net.sourceforge.openutils.mgnlmedia.media.types.impl;

import info.magnolia.cms.beans.runtime.Document;
import info.magnolia.cms.beans.runtime.FileProperties;
import info.magnolia.cms.beans.runtime.MultipartForm;
import info.magnolia.cms.core.Content;
import info.magnolia.cms.core.NodeData;
import info.magnolia.cms.security.AccessDeniedException;
import info.magnolia.cms.util.NodeDataUtil;
import info.magnolia.module.admininterface.SaveHandlerImpl;

import java.io.File;
import java.util.Map;

import javax.jcr.RepositoryException;

import net.sourceforge.openutils.mgnlmedia.media.configuration.MediaConfigurationManager;


/**
 * @author molaschi
 * @version $Id: AudioTypeHandler.java 926 2008-11-10 15:40:59Z fgiust $
 */
public class AudioTypeHandler extends BaseTypeHandler
{

    /**
     * {@inheritDoc}
     */
    @Override
    public void init(Content typeDefinitionNode)
    {

    }

    /**
     * {@inheritDoc}
     */
    @Override
    public String getUrl(Content media)
    {
        NodeData file = media.getNodeData("audio");
        String fileName = file.getAttribute(FileProperties.PROPERTY_FILENAME)
            + "."
            + file.getAttribute(FileProperties.EXTENSION);

        return MediaConfigurationManager.getInstance().getURIMappingPrefix() + media.getHandle() + "/audio/" + fileName;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public String getUrl(Content media, Map<String, String> options)
    {
        return getUrl(media);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public String getFilename(Content media)
    {
        NodeData file = media.getNodeData("audio");
        return file.getAttribute(FileProperties.PROPERTY_FILENAME);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public String getExtension(Content media)
    {
        NodeData file = media.getNodeData("audio");
        return file.getAttribute(FileProperties.PROPERTY_EXTENSION);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public String getFullFilename(Content media)
    {
        return getFilename(media) + "." + getExtension(media);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public String getThumbnailUrl(Content media)
    {
        return "/.resources/media/icons/audioThumb.jpg";
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public String getPreviewUrl(Content media)
    {
        return getUrl(media);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean onPostSave(Content media)
    {
        return super.onPostSave(media);
    }

    public void saveFromZipFile(Content media, File f, String cleanFileName, String extension)
        throws AccessDeniedException, RepositoryException
    {
        Document doc = new Document(f, "audio/" + extension);
        doc.setExtention(extension);
        SaveHandlerImpl.saveDocument(media, doc, "audio", cleanFileName, cleanFileName
            + "_"
            + FileProperties.PROPERTY_TEMPLATE);
        this.onPostSave(media);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public String getNewNodeName(MultipartForm form)
    {
        return form.getParameter("audio_" + FileProperties.PROPERTY_FILENAME);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean onSavingPropertyMedia(Content media, Content parentNode, Content configNode, String name,
        MultipartForm form, int type, int valueType, int isRichEditValue, int encoding) throws RepositoryException,
        AccessDeniedException
    {
        String value = form.getParameter(name);

        NodeData nd = NodeDataUtil.getOrCreate(parentNode, name);
        nd.setValue(value);

        return true;
    }

}
