/**
 * Copyright Openmind http://www.openmindonline.it
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
package net.sourceforge.openutils.mgnlmedia.media.types.impl;

import info.magnolia.cms.beans.runtime.Document;
import info.magnolia.cms.beans.runtime.FileProperties;
import info.magnolia.cms.beans.runtime.MultipartForm;
import info.magnolia.cms.core.Content;
import info.magnolia.cms.core.NodeData;
import info.magnolia.cms.security.AccessDeniedException;
import info.magnolia.cms.util.NodeDataUtil;
import info.magnolia.module.admininterface.SaveHandlerImpl;

import java.io.File;
import java.util.Collection;
import java.util.Map;

import javax.jcr.RepositoryException;

import net.sourceforge.openutils.mgnlmedia.media.configuration.MediaConfigurationManager;
import net.sourceforge.openutils.mgnlmedia.media.utils.ImageUtils;

import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;


/**
 * @author molaschi
 * @version $Id: ImageTypeHandler.java 926 2008-11-10 15:40:59Z fgiust $
 */
public class ImageTypeHandler extends BaseTypeHandler
{

    /**
     * Logger.
     */
    private final Logger log = LoggerFactory.getLogger(ImageTypeHandler.class);

    /**
     * {@inheritDoc}
     */
    @Override
    public void init(Content typeDefinitionNode)
    {

    }

    /**
     * {@inheritDoc}
     */
    @Override
    public String getFilename(Content media)
    {
        NodeData file = media.getNodeData("image");
        return file.getAttribute(FileProperties.PROPERTY_FILENAME);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public String getExtension(Content media)
    {
        NodeData file = media.getNodeData("image");
        return file.getAttribute(FileProperties.PROPERTY_EXTENSION);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public String getFullFilename(Content media)
    {
        return getFilename(media) + "." + getExtension(media);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public String getUrl(Content media)
    {
        NodeData file = media.getNodeData("image");
        String fileName = file.getAttribute(FileProperties.PROPERTY_FILENAME)
            + "."
            + file.getAttribute(FileProperties.EXTENSION);

        return MediaConfigurationManager.getInstance().getURIMappingPrefix() + media.getHandle() + "/image/" + fileName;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public String getUrl(Content media, Map<String, String> options)
    {
        return getUrl(media);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public String getThumbnailUrl(Content media)
    {
        if (!ImageUtils.checkOrCreateResolution(media, "thumbnail"))
        {
            return StringUtils.EMPTY;
        }
        return MediaConfigurationManager.getInstance().getURIMappingPrefix()
            + media.getHandle()
            + "/resolutions/thumbnail/data."
            + ImageUtils.getExtension(media, "thumbnail");
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public String getPreviewUrl(Content media)
    {
        if (!ImageUtils.checkOrCreateResolution(media, "preview"))
        {
            return StringUtils.EMPTY;
        }
        return MediaConfigurationManager.getInstance().getURIMappingPrefix()
            + media.getHandle()
            + "/resolutions/preview/data."
            + ImageUtils.getExtension(media, "preview");
    }

    /**
     * {@inheritDoc}
     */
    @SuppressWarnings("unchecked")
    @Override
    public boolean onPostSave(Content media)
    {
        Collection<NodeData> nds = media.getNodeDataCollection("tags-*");
        for (NodeData nd : nds)
        {
            if (nd.getString() != null)
            {
                try
                {
                    nd.setValue(nd.getString().toLowerCase());
                    nd.save();
                }
                catch (RepositoryException ex)
                {
                    log.warn("Error putting property to lower case", ex);
                }
            }
        }
        return super.onPostSave(media);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void saveFromZipFile(Content media, File f, String cleanFileName, String extension)
        throws AccessDeniedException, RepositoryException
    {
        Document doc = new Document(f, "image/" + extension);
        doc.setExtention(extension);
        SaveHandlerImpl.saveDocument(media, doc, "image", cleanFileName, cleanFileName
            + "_"
            + FileProperties.PROPERTY_TEMPLATE);
        this.onPostSave(media);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public String getNewNodeName(MultipartForm form)
    {
        return form.getParameter("image_" + FileProperties.PROPERTY_FILENAME);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean onSavingPropertyMedia(final Content media, final Content parentNode, final Content configNode,
        final String name, final MultipartForm form, final int type, final int valueType, final int isRichEditValue,
        final int encoding) throws RepositoryException, AccessDeniedException
    {
        String value = form.getParameter(name);

        NodeData nd = NodeDataUtil.getOrCreate(parentNode, name);
        nd.setValue(value);

        return true;
    }
}
