/**
 * Copyright Openmind http://www.openmindonline.it
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
package net.sourceforge.openutils.mgnlmedia.media.types.impl;

import info.magnolia.cms.beans.runtime.MultipartForm;
import info.magnolia.cms.core.Content;
import info.magnolia.cms.core.NodeData;
import info.magnolia.cms.util.NodeDataUtil;

import java.io.File;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Map;
import java.util.UUID;

import javax.jcr.AccessDeniedException;
import javax.jcr.RepositoryException;

import net.sourceforge.openutils.mgnlmedia.media.configuration.MediaConfigurationManager;
import net.sourceforge.openutils.mgnlmedia.media.utils.ImageUtils;

import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;


/**
 * @author molaschi
 * @version $Id: YouTubeVideoTypeHandler.java 926 2008-11-10 15:40:59Z fgiust $
 */
public class YouTubeVideoTypeHandler extends BaseTypeHandler
{

    /**
     * Logger.
     */
    private final Logger log = LoggerFactory.getLogger(YouTubeVideoTypeHandler.class);

    /**
     * {@inheritDoc}
     */
    @Override
    public void init(Content typeDefinitionNode)
    {

    }

    /**
     * {@inheritDoc}
     */
    @Override
    public String getUrl(Content media)
    {
        return media.getNodeData("videoUrl").getString();
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public String getUrl(Content media, Map<String, String> options)
    {
        return getUrl(media);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public String getFilename(Content media)
    {
        return getUrl(media);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public String getExtension(Content media)
    {
        return null;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public String getFullFilename(Content media)
    {
        return null;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public String getThumbnailUrl(Content media)
    {
        if (!ImageUtils.checkOrCreateResolution(media, "thumbnail"))
        {
            return StringUtils.EMPTY;
        }
        return MediaConfigurationManager.getInstance().getURIMappingPrefix()
            + media.getHandle()
            + "/resolutions/thumbnail/data."
            + ImageUtils.getExtension(media, "thumbnail");
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public String getPreviewUrl(Content media)
    {
        return getUrl(media);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean onPostSave(Content media)
    {
        return super.onPostSave(media);
    }

    public void saveFromZipFile(Content media, File f, String cleanFileName, String extension)
        throws AccessDeniedException, RepositoryException
    {

    }

    /**
     * {@inheritDoc}
     */
    @Override
    public String getNewNodeName(MultipartForm form)
    {
        String videoUrl = form.getParameter("videoUrl");
        String id = UUID.randomUUID().toString();
        URL url;
        try
        {
            url = new URL(videoUrl);
            for (String att : StringUtils.split(url.getQuery(), "&"))
            {
                if (att.startsWith("v="))
                {
                    id = StringUtils.substringAfter(att, "v=");
                }
            }
        }
        catch (MalformedURLException e)
        {
            log.error("Error parsing youtube url", e);
        }

        return id;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean onSavingPropertyMedia(Content media, Content parentNode, Content configNode, String name,
        MultipartForm form, int type, int valueType, int isRichEditValue, int encoding) throws RepositoryException,
        AccessDeniedException
    {

        String value = form.getParameter(name);

        NodeData nd = NodeDataUtil.getOrCreate(parentNode, name);
        nd.setValue(value);

        return true;
    }
}
